#!/usr/bin/env python3
import sys
import argparse
from Bio.SeqIO.FastaIO import SimpleFastaParser

"""
split a genome into a set of overlapping segments
adapted from nanopolish_makerange.py https://github.com/jts/nanopolish to CulebrONT 
"""
# DRAFT = snakemake.input.draft
# SEGMENT_LENGTH = snakemake.params.segment_len if not snakemake.params.segment_len == '' else 50000
# OVERLAP_LENGTH = snakemake.params.overlap_len if not snakemake.params.overlap_len == '' else 200
# MIN_SEGMENT_LENGTH = 5 * OVERLAP_LENGTH
# OUTPUTFILE = snakemake.output.segments_list

parser = argparse.ArgumentParser(description='Partition a genome into a set of overlapping segments')
parser.add_argument('--draft', help="draft to segment")
parser.add_argument('--segment-length', type=int, default=50000)
parser.add_argument('--overlap-length', type=int, default=200)
parser.add_argument('--output-file', help="output file")
args = parser.parse_args()

DRAFT = args.draft
SEGMENT_LENGTH = args.segment_length
OVERLAP_LENGTH = args.overlap_length
OUTPUT_FILE = args.output_file
MIN_SEGMENT_LENGTH = 5 * OVERLAP_LENGTH

with open(DRAFT, "r") as draft:
    recs = [(title.split(None, 1)[0], len(seq))
            for title, seq in SimpleFastaParser(draft)]

with open(OUTPUT_FILE, "w") as output:
    for name, length in recs:
        n_segments = (length / SEGMENT_LENGTH) + 1
        start = 0
        while start < length:
            end = start + SEGMENT_LENGTH
            # If this segment will end near the end of the contig, extend it to end
            if length - end < MIN_SEGMENT_LENGTH:
                output.write(f"{name}:{start}-{length-1}\n")
                start = length
            else:
                nlen = end + OVERLAP_LENGTH
                output.write(f"{name}:{start}-{nlen}\n")
                start = end


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
from pathlib import Path
from snakemake.logging import logger
from snakemake.utils import validate
import re
from .global_variables import *
from .snakeWrapper import *


class CulebrONT(SnakeWrapper):
    """
    to read file config
    """

    def __init__(self, workflow, config, ):
        super().__init__(workflow, config)
        # workflow is available only in __init__
        # print("\n".join(list(workflow.__dict__.keys())))
        # print(workflow.__dict__)

        # Initialisation of CulebrONT attributes
        self.assembly_tools_activated = []
        self.polishing_tools_activated = []
        self.correction_tools_activated = []
        self.quality_tools_activated = []
        self.quality_step = []
        self.last_steps_list = []
        self.pipeline_stop = None

        self.fastq_files_list = []
        self.fastq_files_ext = []
        self.fastq_gzip = None

        self.illumina_files_list = []
        self.illumina_files_ext = []
        self.illumina_gzip = None

        self.R1 = ""
        self.R2 = ""

        self.add_circular_name = None
        self.TMP = None
        self.TCM = None
        self.TAG = None

        self.draft_to_correction = None
        self.draft_to_correction_index_fai = None
        self.draft_to_correction_index_mmi = None

        self.nb_racon_rounds = None
        self.nb_pilon_rounds = None

        self.__check_config_dic()
        self.__cleaning_for_rerun()

        # write corrected config to output:
        self.write_config(f"{self.get_config_value('DATA', 'OUTPUT')}/config_corrected.yaml")

        # using schemas to check mandatory value from yaml format
        try:
            validate(self.config, INSTALL_PATH.joinpath("schemas/config.schema.yaml").resolve().as_posix())
        except Exception as e:
            raise ValueError(
                f"{e}\n\nCONFIG FILE CHECKING STRUCTURE FAIL : you need to verify {self.path_config} KEYS:VALUES: {str(e)[30:76]}\n")

    def __split_illumina(self):
        R1 = []
        R2 = []
        for fastq in self.illumina_files_list:
            if '_R1' in fastq:
                R1.append(fastq)
            elif '_R2' in fastq:
                R2.append(fastq)
        return R1, R2

    def __cleaning_for_rerun(self):
        """Cleaning Report and dags if we rerun the snakemake a second time
        TODO: improve check if final report run maybe check html file"""
        if Path(self.get_config_value('DATA', 'OUTPUT')).joinpath("FINAL_REPORT/CulebrONT_report.html").exists():
            logger.warning(
                f"\nWARNING: If you want to rerun culebront a second time please delete the {self.get_config_value('DATA', 'OUTPUT')}FINAL_REPORT and REPORT repertories for each sample\n")

    def __check_tools_config(self, tool, mandatory=()):
        """Check if path is a file if not empty
        :return absolute path file"""
        tool_OK = False
        if tool in ["FLAGSTATS"]:
            tool = "SAMTOOLS"

        def check_singularity(sif):
            level1 = "SINGULARITY"
            path_file = self.tools_config[level1][sif]
            if re.findall("shub://SouthGreenPlatform/CulebrONT_pipeline", path_file, flags=re.IGNORECASE):
                raise ValueError(
                    f'CONFIG FILE CHECKING FAIL : shub download is obsolete')
            else:
                path_file = eval(f"f'{path_file}'")
                path = Path(path_file).resolve().as_posix()
                if path and path_file:
                    if not Path(path).exists() or not Path(path).is_file():
                        raise FileNotFoundError(
                            f'CONFIG FILE CHECKING FAIL : please check tools_config.yaml in the {level1} section, {tool} file "{path}" {"does not exist" if not Path(path).exists() else "is not a valid file"}')
                    else:
                        self.tools_config[level1][tool] = path

        # If only singularity tools
        if not self.use_env_modules and self.use_singularity:
            check_singularity("REPORT")     # check singularity image for Report
            check_singularity("TOOLS")      # check singularity image for conda (contains a lot of conda envs)
            tool_OK = True

        # If only envmodule
        if self.use_env_modules and not self.use_singularity:
            envmodule_key = self.tools_config["ENVMODULE"][tool]
            if not envmodule_key:
                raise ValueError(
                    f'CONFIG FILE CHECKING FAIL : please check tools_config.yaml in the "ENVMODULE" section, {tool} is empty')
            tool_OK = True

        # If envmodule and singularity
        if self.use_env_modules and self.use_singularity:
            raise ValueError(
                f"CONFIG FILE CHECKING FAIL : Use env-module or singularity but don't mix them")

        if len(mandatory) > 0 and not tool_OK:
            raise FileNotFoundError(
                f'CONFIG FILE CHECKING FAIL : please check tools_config.yaml in the  {tool} params, please append Singularity or module load, is mandatory for tool: {" ".join(mandatory)}')

    def __build_tools_activated(self, level1, allow, mandatory=False):
        tools_activate = []
        for tool, activated in self.config[level1].items():
            if tool in allow:
                boolean_activated = var_2_bool(level1, tool, activated)
                if boolean_activated:
                    tools_activate.append(tool)
                    self.set_config_value(level1=level1, level2=tool, value=boolean_activated)
                    self.__check_tools_config(tool, [tool])
            else:
                raise ValueError(f'CONFIG FILE CHECKING FAIL : {level1} {tool} not allow on CulebrONT"')
        if len(tools_activate) == 0 and mandatory:
            raise ValueError(f"CONFIG FILE CHECKING FAIL : you need to set True for at least one {level1} from {allow}")
        return tools_activate

    def __build_quality_step_list(self, only_last=False):
        last_steps_list = []
        suffix = ""
        if bool(self.config["FIXSTART"]):
            suffix = "_STARTFIXED"
        if self.correction_tools_activated:
            step = "CORRECTION"
            for corrector in self.correction_tools_activated:
                last_steps_list.append(f"STEP_{step}_{corrector}{suffix if step in self.pipeline_stop else ''}")
            if only_last:
                return last_steps_list

        if self.polishing_tools_activated:
            step = "POLISHING"
            for polisher in self.polishing_tools_activated:
                last_steps_list.append(f"STEP_{step}_{polisher}{suffix if step in self.pipeline_stop else ''}")
            if only_last:
                return last_steps_list
        if self.assembly_tools_activated:
            step = "ASSEMBLY"
            last_steps_list.append(f"STEP_{step}{suffix if step in self.pipeline_stop else ''}")
            # for assembler in self.assembly_tools_activated:
            # last_steps_list.append(f"STEP_{step}_{assembler}{suffix}" )
            if only_last:
                return last_steps_list
        return last_steps_list

    def __get_last_step(self):
        if self.correction_tools_activated:
            return "CORRECTION"
        if self.polishing_tools_activated:
            return "POLISHING"
        if self.assembly_tools_activated:
            return "ASSEMBLY"

    def __check_config_dic(self):
        """Configuration file checking"""
        # check tools activation
        self.assembly_tools_activated = self.__build_tools_activated("ASSEMBLY", AVAIL_ASSEMBLY, True)
        self.polishing_tools_activated = self.__build_tools_activated("POLISHING", AVAIL_POLISHING)
        self.correction_tools_activated = self.__build_tools_activated("CORRECTION", AVAIL_CORRECTION)
        self.quality_tools_activated = self.__build_tools_activated("QUALITY", AVAIL_QUALITY)
        self.pipeline_stop = self.__get_last_step()

        # check mandatory directory
        self._check_dir_or_string(level1="DATA", level2="OUTPUT")
        self._check_dir_or_string(level1="DATA", level2="FASTQ", mandatory=self.assembly_tools_activated)

        # check if fastq file for assembly
        self.fastq_files_list, fastq_files_list_ext = get_files_ext(self.get_config_value('DATA', 'FASTQ'),
                                                                    ALLOW_FASTQ_EXT)
        if not self.fastq_files_list:
            raise ValueError(
                f"CONFIG FILE CHECKING FAIL : you need to append at least on fastq with extension on {ALLOW_FASTQ_EXT}")
        # check if all fastq have the same extension
        if len(fastq_files_list_ext) > 1:
            raise ValueError(
                f"CONFIG FILE CHECKING FAIL : Please use only the same format for assembly FASTQ data, not: {fastq_files_list_ext}")
        else:
            self.fastq_files_ext = fastq_files_list_ext[0]
        # check if fastq are gzip
        if "gz" in self.fastq_files_ext:
            self.fastq_gzip = True

        # CHECK ILLUMINA
        need_illumina = [var_2_bool("QUALITY", "KAT", self.get_config_value("QUALITY", "KAT")),
                         var_2_bool("CORRECTION", "PILON", self.get_config_value("CORRECTION", "PILON")),
                         var_2_bool("QUALITY", "FLAGSTATS", self.get_config_value("QUALITY", "FLAGSTATS"))]
        if True in need_illumina:
            self._check_dir_or_string(level1="DATA", level2="ILLUMINA", mandatory=["KAT", "PILON", "FLAGSTATS"])
            self.illumina_files_list, illumina_files_list_ext = get_files_ext(self.get_config_value('DATA', 'ILLUMINA'),
                                                                              ALLOW_FASTQ_EXT)

            if not self.illumina_files_list:
                raise ValueError(
                    f"CONFIG FILE CHECKING FAIL : you need to append at least on fastq illumina with extension on {ALLOW_FASTQ_EXT}")
            # check if all fastq have the same extension
            if len(illumina_files_list_ext) > 1:
                raise ValueError(
                    f"CONFIG FILE CHECKING FAIL : Please use only the same format for ILLUMINA FASTQ data, not: {illumina_files_list_ext}")
            else:
                self.illumina_files_ext = illumina_files_list_ext[0]
            # check if fastq are gzip
            if "gz" in self.illumina_files_ext:
                self.illumina_gzip = True

            self.R1, self.R2 = self.__split_illumina()

        # build quality_step and last_steps_list; function test if FIXSTART to append only to last step
        self.last_steps_list = self.__build_quality_step_list(only_last=True)
        self.quality_step = self.__build_quality_step_list(only_last=False)

        # check files if QUAST
        if bool(self.config["QUALITY"]["QUAST"]):
            self._check_file_or_string(level1="DATA", level2="REF")
        genome_pb = convert_genome_size(self.get_config_value('DATA', 'GENOME_SIZE'))
        self.set_config_value(level1="params", level2="QUAST", level3="GENOME_SIZE_PB", value=genome_pb)

        # check files if MAUVE
        if bool(self.config["MSA"]["MAUVE"]):
            self._check_file_or_string(level1="DATA", level2="REF", mandatory=["MAUVE"])
            # Make sure running Mauve makes sense
            if len(self.assembly_tools_activated) < 2 and len(self.quality_step) < 2:
                raise ValueError(
                    "CONFIG FILE CHECKING ERROR : MAUVE is irrelevant if you have a single final assembly as your config file implies (need more than one assembler and/or a correction step). Mauve will not be run !! ")

        # check BUSCO database if activate
        if bool(self.config["QUALITY"]["BUSCO"]):
            self._check_dir_or_string(level1='params', level2='BUSCO', level3='DATABASE', mandatory=["BUSCO"], check_string=True)

        # check DIAMOND database if activate
        if bool(self.config["QUALITY"]["BLOBTOOLS"]):
            self._check_file_or_string(level1='params', level2='DIAMOND', level3='DATABASE', mandatory=["BLOBTOOLS", 'DIAMOND'])

        # check if NANOPOLISH activate, if true compare fastq and fast5 files
        if "NANOPOLISH" in self.correction_tools_activated:
            self._check_dir_or_string(level1="DATA", level2="FAST5", mandatory=["NANOPOLISH"])
            fast5_dir_list = get_dir(self.config['DATA']['FAST5'])
            fastq_files_list, _ = get_files_ext(self.config['DATA']['FASTQ'], ALLOW_FASTQ_EXT, add_ext=False)

            if set(fastq_files_list) - set(fast5_dir_list):
                raise ValueError(
                    f"CONFIG FILE CHECKING ERROR : You don't have a fast5 repository for each of your fastq file (they should have the same name). This can raise a problem if you choose to use Nanopolish. Please check your data.:\n\t- fast5_dir_list:{fast5_dir_list}\n\t- fastq_files_list: {fastq_files_list}\n\n")

        # check Medaka config if activate
        if bool(self.config['CORRECTION']['MEDAKA']):
            if not bool(self.config['params']['MEDAKA']['MEDAKA_TRAIN_WITH_REF']):
                self._check_file_or_string(level1='params', level2='MEDAKA', level3='MEDAKA_MODEL_PATH', mandatory=["MEDAKA"], check_string=True)
            else:
                self._check_file_or_string(level1='DATA', level2='REF', mandatory=["MEDAKA"])

        # Check racon round
        if bool(self.config['POLISHING']['RACON']) and not (0 < int(self.config['params']['RACON']['RACON_ROUNDS']) < 10):
            raise ValueError(f"CONFIG FILE CHECKING ERROR : You have activated RACON, but RACON_ROUNDS is invalid, 0 < RACON_ROUNDS={self.config['params']['RACON']['RACON_ROUNDS']} < 10 . \n")

        ##############################
        # check workflow compatibility
        if not bool(self.config['POLISHING']['RACON']) and bool(self.config['ASSEMBLY']['MINIASM']):
            logger.warning(
                f"\nWARNING: RACON is automatically launched (2 rounds by default) for minipolish if MINIASM is activated !! . \n")

        # check size of genome
        if int(convert_genome_size(self.get_config_value('DATA', 'GENOME_SIZE'))) >= 50000000 and self.get_config_value('MSA', 'MAUVE'):
            logger.warning(
                f"WARNING: CONFIG FILE CHECKING WARNING : MAUVE if fixed to FALSE because genome size  >= 50 mb !! \n")
            self.set_config_value('MSA', 'MAUVE', False)

        # Make sure running fixstart makes sense
        if not bool(self.config['CIRCULAR']) and bool(self.config['FIXSTART']):
            raise ValueError(
                f"CONFIG FILE CHECKING ERROR : FIXSTART is irrelevant if you have not activated CIRCULAR. FIXSTART will not be run !! \n")

        # if you want to run mauve fixstart has to be activated
        if bool(self.config['CIRCULAR']) and not bool(self.config['FIXSTART']):
            raise ValueError(
                f"CONFIG FILE CHECKING ERROR : FIXSTART must be activated if CIRCULAR is TRUE on config file.  !! \n")

        # check for BLOBTOOLS
        if bool(self.config["QUALITY"]["BLOBTOOLS"]):
            self._check_file_or_string(level1='params', level2='BLOBTOOLS', level3='NAMES', mandatory=["BLOBTOOLS"])
            self._check_file_or_string(level1='params', level2='BLOBTOOLS', level3='NODES', mandatory=["BLOBTOOLS"])

        #############################################
        # Build variables name for files
        if bool(self.config['CIRCULAR']):
            self.add_circular_name = "CIRCULARISED"
            self.TMP = "TMP"
            self.TCM = "FALSE"
        else:
            self.add_circular_name = ""
            self.TMP = ""
            self.TCM = ""

        if not bool(self.config['POLISHING']['RACON']) and bool(self.config['CIRCULAR']):
            self.TAG = "TMPTAG"
        else:
            self.TAG = ""

        # ############################### DEF ####################################
        self.nb_racon_rounds = '2' if not bool(self.config['POLISHING']['RACON']) else str(
            self.config['params']['RACON']['RACON_ROUNDS'])
        self.nb_pilon_rounds = '1' if not bool(self.config['CORRECTION']['PILON']) else str(
            self.config['params']['PILON']['PILON_ROUNDS'])



from pathlib import Path

DOCS = "https://culebront-pipeline.readthedocs.io/en/latest/"
GIT_URL = "https://github.com/SouthGreenPlatform/culebrONT"

INSTALL_PATH = Path(__file__).resolve().parent
SINGULARITY_URL_FILES = [('https://itrop.ird.fr/culebront_utilities/singularity_build/Singularity.culebront_tools.sif',
                          f'{INSTALL_PATH}/containers/Singularity.culebront_tools.sif'),
                         ('https://itrop.ird.fr/culebront_utilities/singularity_build/Singularity.report.sif',
                          f'{INSTALL_PATH}/containers/Singularity.report.sif')
                         ]

DATATEST_URL_FILES = ("Data-Xoo-sub.zip", "https://itrop.ird.fr/culebront_utilities/Data-Xoo-sub.zip")


AVAIL_ASSEMBLY = ("CANU", "FLYE", "MINIASM", "RAVEN", "SMARTDENOVO", "SHASTA")
AVAIL_CORRECTION = ("NANOPOLISH", "MEDAKA", "PILON")
AVAIL_POLISHING = ("RACON")
AVAIL_QUALITY = ("BUSCO", "QUAST", "BLOBTOOLS", "ASSEMBLYTICS", "KAT", "FLAGSTATS")

ALLOW_FASTQ_EXT = (".fastq", ".fq", ".fq.gz", ".fastq.gz")




#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# @package controllingMapping.py
# @author Francois Sabot

"""
	Controlling mapping data for a de novo assembly
	==========================
	:author:  François Sabot (thanks to Julie Orjuela Scripts help)
	:contact: francois.sabot@ird.fr
	:date: 04/04/2019
	:version: 0.1
	Script description
	------------------
	controllingMapping.py will take a BAM file issued from the mapping of Illumina data on a de novo assembly and try to catch the misassembled contigs based on local depth and local mapping errors
	-------
	>>> controllingMapping.py -i mapping.bam -o outputPrefix [-s windowSize -t tempLocation]

"""

##################################################
## Modules
##################################################
import sys, os, subprocess, re
current_dir = os.path.dirname(os.path.abspath(__file__))+"/"

## Python modules
import click
from pathlib import Path
from datetime import datetime
import pysam
import numpy as np
import pandas as pd
import matplotlib.pyplot as plt

##################################################
## Variables Globales
##################################################
version="0.1"
VERSION_DATE='04/04/2018'
debug="False"
#debug="True"

##################################################
## Functions
##################################################
def multicov(bedFile,bamFile,optionsList=("")):
	'''Commentaires'''
	optionsTxt = "".join(optionsList)
	command = "bedtools multicov " + optionsTxt + " -bams " + bamFile + " -bed " + bedFile
	#print (command)
	try:
		optionsTxt = str(OptionsTxt)
		resultCom = os.popen(command,"r")
	except NameError:
		resultCom = os.popen(command,"r")
	
	hashCov = {}
	for line in resultCom:
		#print(line)
		mainLine = line.strip()
		fields = mainLine.split("\t")
		start = int(fields[1])
		stop = int(fields[2])
		position = start + int(stop - start/2)
		try:
			hashCov[fields[0]].append((position,int(fields[3])))
		except KeyError:
			hashCov[fields[0]] = [(position,int(fields[3]))]
		#break
		#sys.exit()
	return hashCov	


@click.command("controlling_mapping", short_help='This Program identifies the misassemblies in contigs using BAM, and uses BedTools2',
			   context_settings={'help_option_names': ('-h', '--help'),"max_content_width":800}, no_args_is_help=True)
@click.version_option(version, '--version', '-v')
@click.option('--input', '-i', type=click.Path(exists=True, file_okay=True, dir_okay=False, readable=True, resolve_path=True), required=True, help='BAM file issued from mapping (must be indexed)')
@click.option('--out', '-o', type=click.Path(exists=False, file_okay=True, dir_okay=False, readable=True, resolve_path=True), required=True, help='Prefix for the output files')
@click.option('--size', '-s', type=int, required=False, default=1000, show_default=True, help='window scan size in bases')
@click.option('--temp', '-t', type=click.Path(exists=True, file_okay=False, dir_okay=True, readable=True, resolve_path=True), required=False, default="/tmp", show_default=True, help='Location of temp directory')
def main(input, out, size, temp):
	"""
	\b
	This Program identifies the misassemblies in contigs using BAM, and uses BedTools2
	controllingMapping.py will take a BAM file issued from the mapping of Illumina data on a de novo assembly
	and try to catch the misassembled contigs
	based on local depth and local mapping errors
	"""
	inputFile = input
	outputPrefix = out
	windowSize = size
	tempLocation = temp
	start_time = datetime.now()

	# Welcome message
	click.secho(f"""{"#" * 80}\n#{Path(sys.argv[0]).stem + " " + version:^78}#\n{"#" * 80}\nStart time: {start_time:%d-%m-%Y at %H:%M:%S}\nCommande line run: {" ".join(sys.argv)}\n""", fg="green")

	# open output handle
	outputFile = Path(outputPrefix).joinpath("_position.csv")
	outputHandle = open(outputFile, "w")

	# PySam import
	bamFile = pysam.AlignmentFile(inputFile, "r");

	# Picking up infos on sequences
	sizes = bamFile.lengths
	names = bamFile.references
	seqInfo = {a: b for a, b in zip(names, sizes)}
	bamFile.close()

	# Temp bed opening
	with open(tempLocation + "/temp.bed", "w") as tempHandle:
		for seq in seqInfo.keys():
			for window in range(1, seqInfo[seq], windowSize):
				start = str(window)
				stop = window + windowSize
				if stop > seqInfo[seq]:
					stop = seqInfo[seq]
				stop = str(stop)
				tempHandle.write(seq + "\t" + start + "\t" + stop + "\n")

	depthTotal = multicov(tempBed, inputFile)
	options = ("-p")
	depthOk = multicov(tempBed, inputFile, options)

	# Creating the dataframe
	for contigs in depthTotal.keys():
		depthInfo = {a: b for a, b in depthTotal[contigs]}
		okInfo = {a: b for a, b in depthOk[contigs]}
		globalInfo = {}
		for position in depthInfo.keys():
			globalInfo[position] = [depthInfo[position], okInfo[position]]
		df = pd.DataFrame.from_dict(globalInfo, orient='index', columns=['Depth', 'Ok'])
		df = df.sort_index()
		# print(df)
		somme = df['Depth'].sum()
		if somme > 0:
			plt.figure()
			ax = plt.gca()
			df.plot(kind='line', y='Depth', ax=ax, title=contigs)
			df.plot(kind='line', y='Ok', color='red', ax=ax)
			filename = contigs + ".png"
			plt.savefig(filename)
		else:
			continue

	sys.exit()
##################################################
## Main code
##################################################
if __name__ == "__main__":
	main()



import os
import re
import subprocess
import urllib.request
from shutil import rmtree
import click
from tqdm import tqdm
from .global_variable import *


def check_privileges():
    if not os.environ.get("SUDO_UID") and os.geteuid() != 0:
        click.secho(f"\n    ERROR : You need to run -r, --restore with sudo privileges or as root\n", fg="red")
    else:
        return True


class DownloadProgressBar(tqdm):
    def update_to(self, b=1, bsize=1, tsize=None):
        if tsize is not None:
            self.total = tsize
        self.update(b * bsize - self.n)


def download_url(tuple_value):
    url, output_path = tuple_value
    with DownloadProgressBar(unit='B', unit_scale=True,
                             miniters=1, desc=url.split('/')[-1]) as t:
        urllib.request.urlretrieve(url, filename=output_path, reporthook=t.update_to)


def multiprocessing_download(urls_list, threads = 2):
    from multiprocessing.pool import ThreadPool
    return ThreadPool(threads).imap_unordered(download_url, urls_list)


INSTALL_PATH = Path(__file__).resolve().parent.parent


def package_name():
    return Path(INSTALL_PATH).stem


def get_install_mode():
    """detect install mode"""
    if INSTALL_PATH.joinpath(".mode.txt").exists():
        return INSTALL_PATH.joinpath(".mode.txt").open("r").readline().strip()
    else:
        return "notInstall"


def get_version():
    """Read VERSION file to know current version
    Returns:
        version: actual version read on the VERSION file
    Examples:
        version = get_version()
        print(version)
            1.3.0
    """
    with open(INSTALL_PATH.joinpath("VERSION"), 'r') as version_file:
        return version_file.readline().strip()


def get_last_version(url, current_version):
    """Function for know the last version of Git repo in website"""
    try:
        from urllib.request import urlopen
        from re import search
        import click
        module_mane = url.split('/')[-1]
        HTML = urlopen(f"{url}/tags").read().decode('utf-8')
        str_search = f"{url.replace('https://github.com', '')}/releases/tag/.*"
        lastRelease = search(str_search, HTML).group(0).split("/")[-1].split('"')[0]
        epilogTools = ""
        if str(current_version) != lastRelease:
            if lastRelease < str(current_version):
                epilogTools = click.style(f"\n    ** NOTE: This {module_mane} version ({current_version}) is higher than the production version ({lastRelease}), you are using a dev version\n\n", fg="yellow", bold=True)
            elif lastRelease > str(current_version):
                epilogTools = click.style(f"\n    ** NOTE: The Latest version of {module_mane} {lastRelease} is available at {url}/releases\n\n",fg="yellow", underline=True)
        return epilogTools
    except Exception as e:
        epilogTools = click.style(f"\n    ** ENABLE TO GET LAST VERSION, check internet connection\n{e}\n\n", fg="red")
        return epilogTools


def command_required_option_from_option(require_name, require_map):
    import click

    class CommandOptionRequiredClass(click.Command):
        def invoke(self, ctx):
            require = ctx.params[require_name]
            if require not in require_map:
                raise click.ClickException(click.style(f"Unexpected value for --'{require_name}': {require}\n", fg="red"))
            if ctx.params[require_map[require].lower()] is None:
                raise click.ClickException(click.style(f"With {require_name}={require} must specify option --{require_map[require]} path/to/modules\n", fg="red"))
            super(CommandOptionRequiredClass, self).invoke(ctx)
    return CommandOptionRequiredClass


def get_dir(path):
    """List of directory included on folder"""
    return [elm.name for elm in Path(path).glob("*") if elm.is_dir()]


def get_files_ext(path, extensions, add_ext=True):
    """List of files with specify extension included on folder

    Arguments:
        path (str): a path to folder
        extensions (list or tuple): a list or tuple of extension like (".py")
        add_ext (bool): if True (default), file have extension

    Returns:
        :class:`list`: List of files name with or without extension , with specify extension include on folder
        :class:`list`: List of  all extension found

     """
    if not (extensions, (list, tuple)) or not extensions:
        raise ValueError(f'ERROR: "extensions" must be a list or tuple not "{type(extensions)}"')
    tmp_all_files = []
    all_files = []
    files_ext = []
    for ext in extensions:
        tmp_all_files.extend(Path(path).glob(f"**/*{ext}"))

    for elm in tmp_all_files:
        ext = "".join(elm.suffixes)
        if ext not in files_ext:
            files_ext.append(ext)
        if add_ext:
            all_files.append(elm.as_posix())
        else:
            if len(elm.suffixes) > 1:

                all_files.append(Path(elm.stem).stem)
            else:
                all_files.append(elm.stem)
    return all_files, files_ext


def var_2_bool(key, tool, to_convert):
    """convert to boolean"""
    if isinstance(type(to_convert), bool):
        return to_convert
    elif f"{to_convert}".lower() in ("yes", "true", "t"):
        return True
    elif f"{to_convert}".lower() in ("no", "false", "f"):
        return False
    else:
        raise TypeError(
            f'CONFIG FILE CHECKING FAIL : in the "{key}" section, "{tool}" key: "{to_convert}" is not a valid boolean')


def create_bash_completion():
    bashrc_file = Path("~/.bashrc").expanduser().as_posix()
    output = subprocess.run(
        ["bash", "-c", "echo ${BASH_VERSION}"], stdout=subprocess.PIPE
    )
    match = re.search(r"^(\d+)\.(\d+)\.\d+", output.stdout.decode())
    if match is not None:
        major, minor = match.groups()
    if major < "4" or (major == "4" and minor < "4"):
        click.secho(f"\n    WARNING Shell completion is not supported for Bash versions older than 4.4.", fg="red", nl=False)
        # raise RuntimeError("Shell completion is not supported for Bash versions older than 4.4.")
    else:
        if not Path(f"{INSTALL_PATH}/{package_name()}-complete.sh").exists():
            build_completion = f"_{package_name().upper()}_COMPLETE=bash_source {package_name()} > {INSTALL_PATH}/{package_name()}-complete.sh"
            os.system(build_completion)
            path_bash = None
            with open(bashrc_file, "r") as bash_file_read:
                for line in bash_file_read:
                    if f"{package_name().upper()}" in line:
                        path_bash = bash_file_read.readline().strip()
            if path_bash:
                load = f"{path_bash[2:]}"
                if f"{INSTALL_PATH}/{package_name()}-complete.sh" != load:
                    click.secho(f"\n    WARNING autocompletion for {package_name().upper()} already found on {bashrc_file}, with other path please fix the good:", fg="red", nl=False)
                    click.secho(f"\n    Load on bashrc: {load}\n    New install:    {INSTALL_PATH}/{package_name()}-complete.sh", fg='bright_red')
            else:
                with open(bashrc_file, "a") as bash_file_open:
                    append_bashrc = f"\n#Add autocompletion for {package_name().upper()}\n. {INSTALL_PATH}/{package_name()}-complete.sh"
                    bash_file_open.write(append_bashrc)
                    click.secho(f"\n    INSTALL autocompletion for {package_name().upper()} on {bashrc_file} with command {append_bashrc}\nUpdate with command:\nsource ~/.bashrc", fg="yellow")


def create_envmodules(modules_dir):
    from .. import MODULE_FILE, __version__
    modules_dir = Path(modules_dir)
    modules_dir.mkdir(parents=True, exist_ok=True)
    modules_dir.joinpath(f"{__version__}").open("w").write(MODULE_FILE)
    click.edit(require_save=False, extension='', filename=modules_dir.joinpath(f"{__version__}").as_posix())
    click.secho(f"\n    Success install module file for version {__version__} on path '{modules_dir}'", fg="yellow")


def clean_home():
    if Path(f"~/.config/{package_name()}/").expanduser().exists():
        rmtree(Path(f"~/.config/{package_name()}/").expanduser().as_posix())


def check_and_download_singularity():
    # check if already download if true pop from list
    from . import SINGULARITY_URL_FILES, INSTALL_PATH
    SINGULARITY_URL_FILES_DOWNLOAD = []
    for imgs_list in SINGULARITY_URL_FILES:
        url, path_install = imgs_list
        if not Path(path_install).exists():
            SINGULARITY_URL_FILES_DOWNLOAD.append(imgs_list)
        else:
            click.secho(f"    File: {path_install} already download, done.", fg="yellow", nl=True)
    results = multiprocessing_download(SINGULARITY_URL_FILES_DOWNLOAD)
    for r in results:
        click.secho(r, fg="blue")
    click.secho(f"\n    WARNING please check if binding is active on your singularity configuration, see https://sylabs.io/guides/3.0/user-guide/bind_paths_and_mounts.html", fg = "bright_red")


def convert_genome_size(size):
    import re
    mult = dict(K=10 ** 3, M=10 ** 6, G=10 ** 9, T=10 ** 12, N=1)
    search = re.search(r'^(\d+\.?\d*)\s*(.*)$', size)
    if not search or len(search.groups()) != 2:
        raise ValueError(
            f"CONFIG FILE CHECKING FAIL : not able to convert genome size please only use int value with{' '.join(mult.keys())} upper or lower, N or empty is bp size")
    else:
        value, unit = search.groups()
        if not unit:
            return int(value)
        elif unit and unit.upper() not in mult.keys():
            raise ValueError(
                f"CONFIG FILE CHECKING FAIL : '{unit}' unit value not allow or not able to convert genome size please only use int value with{' '.join(mult.keys())} upper or lower, N or empty is bp size")
        else:
            return int(float(value) * mult[unit.upper()])




import click
from click import Abort
from shutil import rmtree, unpack_archive
import subprocess
from cookiecutter.main import cookiecutter
from culebrONT.snakeWrapper import *

required_options = {
    True: 'modules_dir',
    False: 'scheduler'
}


@click.command("install_cluster", short_help=f'Install {package_name()} on HPC cluster',
               context_settings=dict(max_content_width=800),
               cls=command_required_option_from_option('create_envmodule', required_options), no_args_is_help=True)
@click.option('--scheduler', '-s', default="slurm", type=click.Choice(['slurm'], case_sensitive=False),
              prompt='Choose your HPC scheduler', show_default=True, help='Type the HPC scheduler (for the moment, only slurm is available ! )')
@click.option('--env', '-e', default="modules", type=click.Choice(['modules', 'singularity'], case_sensitive=False),
              prompt='Choose mode for tools dependencies', show_default=True, help='Mode for tools dependencies ')
@click.option('--bash_completion/--no-bash_completion', is_flag=True, required=False, default=True, show_default=True,
              help=f'Allow bash completion of {package_name()} commands on the bashrc file')
@click.option('--create_envmodule/--no-create_enmodules', is_flag=True, required=False, default=False,
              show_default=True,
              help=f'Create a env module file allowing load {package_name()} in a cluster')
@click.option('--modules_dir', '-m', default=None,
              type=click.Path(exists=False, dir_okay=True, file_okay=False, readable=True, resolve_path=True),
              required=False, show_default=True,
              help='Directory used to save the module created by --create_envmodule parameter', is_eager=True)
def install_cluster(scheduler, env, bash_completion, create_envmodule, modules_dir):
    """Run installation for HPC cluster"""
    # rm previous install (ie @julie cluster then local)
    clean_home()
    # build default profile path
    default_profile = DEFAULT_PROFILE

    def fail():
        rmtree(default_profile, ignore_errors=True)
        INSTALL_MODE.unlink(missing_ok=True)
        click.secho(f"\n    INSTALL FAIL remove already install files: {default_profile} ", fg="red", err=True)
        raise SystemExit

    # test if install already run
    try:
        if default_profile.exists() and click.confirm(
                click.style(f'    Profile "{default_profile}" exist do you want to remove and continue?\n\n', fg="red"),
                default=False, abort=True):
            rmtree(default_profile, ignore_errors=True)
        default_profile.mkdir(exist_ok=True)

        default_cluster = INSTALL_PATH.joinpath("install_files", f"cluster_config_{scheduler.upper()}.yaml").open(
            "r").read()
        command = r"""sinfo -s | grep "*" | cut -d"*" -f1 """
        default_partition = subprocess.check_output(command, shell=True).decode("utf8").strip()
        if not default_partition:
            click.secho("    Error: Slurm was not found on your system !!", fg="red", err=True)
            fail()
            raise SystemExit
        default_profile.joinpath("cluster_config.yaml").open("w").write(
            default_cluster.replace("PARTITION", default_partition))

        # Download cookiecutter of scheduler
        click.secho(
            f"    You choose '{scheduler}' as scheduler. Download cookiecutter:\n    "
            f"https://github.com/Snakemake-Profiles/{scheduler.lower()}.git",
            fg="yellow")
        cookiecutter(f'https://github.com/Snakemake-Profiles/{scheduler.lower()}.git',
                     checkout=None,
                     no_input=True,
                     extra_context={"profile_name": f'',
                                    "sbatch_defaults": "--export=ALL",
                                    "cluster_config": "$HOME/.config/culebrONT/cluster_config.yaml",
                                    "advanced_argument_conversion": 1,
                                    "cluster_name": ""
                                    },
                     replay=False, overwrite_if_exists=True,
                     output_dir=f'{default_profile}', config_file=None,
                     default_config=False, password=None, directory=None, skip_if_file_exists=True)

        try:
            # default slurm cookiecutter not contain all snakemake variables
            if scheduler == "slurm":
                default_slurm = 'restart-times: 0\njobscript: "slurm-jobscript.sh"\ncluster: ' \
                                '"slurm-submit.py"\ncluster-status: "slurm-status.py"\nmax-jobs-per-second: ' \
                                '1\nmax-status-checks-per-second: 10\nlocal-cores: 1\njobs: 200\nlatency-wait: ' \
                                '1296000\n'
                extra_slurm = f"use-envmodules: {'true' if env == 'modules' else 'false'}\nuse-singularity: " \
                              f"{'true' if env == 'singularity' else 'false'}\nrerun-incomplete: " \
                              f"true\nprintshellcmds: true"
                with open(f"{default_profile}/config.yaml", "w") as config_file:
                    config_file.write(f"{default_slurm}{extra_slurm}")
            click.secho(f"\n    Profile is success install on {default_profile}", fg="yellow")
        except Abort:
            fail()
    except Abort:
        click.secho(f"\n    Profile is already created, skipping {default_profile}", fg="yellow")
    except Exception as e:
        print(e)
        fail()
    try:
        # if singularity activation
        if env == 'singularity':
            # check if already download
            check_and_download_singularity()
        # if envmodule activation
        if create_envmodule:
            create_envmodules(modules_dir)
        # export to add bash completion
        if bash_completion:
            create_bash_completion()
        click.secho(f"\n    Congratulations, you have successfully installed {package_name()} !!!\n\n", fg="green", bold=True)
        INSTALL_MODE.open("w").write("cluster")
    except Exception as e:
        click.secho(f"\n    ERROR : an error was detected, please check {e}", fg="red")
        fail()


@click.command("install_local", short_help=f'Install {package_name()} on local computer',
               context_settings=dict(max_content_width=800), no_args_is_help=False)
@click.option('--bash_completion/--no-bash_completion', is_flag=True, required=False, default=True, show_default=True,
              help=f'Allow bash completion of {package_name()} commands on the bashrc file')
def install_local(bash_completion):
    """
    \b
    Run installation for local computer downloading singularity images automatically.
    """
    # rm previous install (ie @julie cluster then local)
    clean_home()
    # add path to download
    INSTALL_PATH.joinpath("containers").mkdir(exist_ok=True, parents=True)
    try:
        check_and_download_singularity()
        # export to add bash completion
        if bash_completion:
            create_bash_completion()
        # Edit tools to write good path for snakefile
        git_tools_file = GIT_TOOLS_PATH.open("r").read()
        USER_TOOLS_PATH.parent.mkdir(exist_ok=True, parents=True)
        USER_TOOLS_PATH.open("w").write(git_tools_file.replace("{INSTALL_PATH}", f"{INSTALL_PATH}"))
        # good install
        click.secho(f"\n    Congratulations, you have successfully installed {package_name()} !!!\n\n", fg="green", bold=True)
        INSTALL_MODE.open("w").write("local")
    except Exception as e:
        INSTALL_MODE.unlink(missing_ok=True)
        click.secho(f"\n    ERROR : an error was detected, please check {e}", fg="red")
        raise SystemExit


@click.command("test_install", short_help=f'Test {package_name()} with data_test after install (local or cluster)',
               context_settings=dict(max_content_width=800), no_args_is_help=False)
@click.option('--data_dir', '-d', default=None,
              type=click.Path(exists=False, file_okay=False, dir_okay=True, readable=True, resolve_path=True),
              required=True, show_default=False, help='Path to download data test and create config.yaml to run test')
def test_install(data_dir):
    """Test_install fonction downloads a scaled data test, writes a configuration file adapted to it and proposes a command line already to run !!!"""
    # create dir test and configure config.yaml
    data_dir = Path(data_dir).resolve()
    click.secho(f"\n    Created data test dir {data_dir}\n", fg="yellow")
    data_dir.mkdir(parents=True, exist_ok=True)

    data_config_path = data_dir.joinpath("data_test_config.yaml")
    click.secho(f"    Created config file to run data test: {data_config_path}\n", fg="yellow")
    txt = GIT_CONFIG_PATH.open("r").read().replace("DATA_DIR", f"{data_dir}").replace(
        f"./{package_name()}_OUTPUT/", f"{data_dir}/{package_name()}_OUTPUT/")
    data_config_path.open("w").write(txt)

    # download data
    download_zip = data_dir.joinpath(DATATEST_URL_FILES[0])
    if not download_zip.exists():
        click.secho(f"    Download data test\n", fg="yellow")
        results = multiprocessing_download(
            [(DATATEST_URL_FILES[1], download_zip.as_posix())], threads=1)
        for r in results:
            click.secho(r, fg="blue")
        click.secho(f"    Extract archive {download_zip} to {data_dir.as_posix()}\n", fg="yellow")
        unpack_archive(download_zip.as_posix(), data_dir.as_posix())
        download_zip.unlink()

    # build command line
    click.secho(f"    Write command line to run workflow on data test:\n", fg="yellow")
    mode = get_install_mode()
    cmd = f"\n    {package_name()} {'run_cluster' if mode == 'cluster' else 'run_local --threads 1'} --config {data_config_path}\n\n"
    click.secho(cmd, fg='bright_blue')



from pathlib import Path
from .global_variable import *
from .usefull_function import get_install_mode
from snakemake.io import load_configfile
import yaml
from collections import OrderedDict
import pprint


class SnakeWrapper(object):
    """ test generic wrapper class"""
    def __init__(self, workflow, config):
        # workflow is available only in __init__
        # print("\n".join(list(workflow.__dict__.keys())))
        # print(workflow.__dict__)
        self.snakefile = SNAKEFILE
        self.tools_config = None
        self.path_config = workflow.overwrite_configfiles[0]
        self.config = config

        self.use_env_modules = workflow.use_env_modules
        self.use_conda = workflow.use_conda
        self.use_singularity = workflow.use_singularity
        # test if cluster_config.yaml pass to snakemake
        if not workflow.overwrite_clusterconfig and get_install_mode() == "cluster":
            self.cluster_config = load_configfile(DEFAULT_PROFILE.joinpath("cluster_config.yaml"))
        elif not workflow.overwrite_clusterconfig and get_install_mode() == "local":
            self.cluster_config = None
        else:
            self.cluster_config = workflow.overwrite_clusterconfig

        self.load_tool_configfile()

    def load_tool_configfile(self):
        """Test path of tools_path.yaml on default install path, home or argument"""
        if USER_TOOLS_PATH.exists() and not ARGS_TOOLS_PATH.exists():
            self.tools_config = load_configfile(USER_TOOLS_PATH)
        elif ARGS_TOOLS_PATH.exists():
            self.tools_config = load_configfile(ARGS_TOOLS_PATH)
            ARGS_TOOLS_PATH.unlink()
        else:
            self.tools_config = load_configfile(GIT_TOOLS_PATH)

    def get_config_value(self, level1, level2, level3=None):
        """get value on config_file"""
        # TODO add type_value to load a good type
        if level3:
            return self.config[level1][level2][level3]
        elif level2:
            return self.config[level1][level2]
        else:
            return self.config[level1]

    def set_config_value(self, level1, level2, value, level3=None):
        if level3:
            self.config[level1][level2][level3] = value
        else:
            self.config[level1][level2] = value

    def write_config(self, path):
        p = Path(path).parent
        p.mkdir(parents=True, exist_ok=True)
        with open(path, "w") as config_open:
            config_open.write(self.export_use_yaml)

    def _check_dir_or_string(self, level1, level2, mandatory=(), level3=None, check_string=False):
        """ similar to check_dir"""
        path_value = self.get_config_value(level1=level1, level2=level2, level3=level3)
        if path_value:
            path = Path(path_value).resolve().as_posix() + "/"
            # it is a path
            if path_value != "" and "/" in path_value:
                if (not Path(path).exists() or not Path(path).is_dir()) and level2 not in ["OUTPUT"]:
                    raise NotADirectoryError(
                        f'CONFIG FILE CHECKING FAIL : in section:{level1}, {f"subsection:{level2} directory:{level3}" if level3 else f"directory:{level3}"}, "{path}" {"does not exist" if not Path(path).exists() else "is not a valid directory"}')
                else:
                    self.set_config_value(level1=level1, level2=level2, level3=level3, value=path)
            # it is not a path
            elif path_value and "/" not in path_value and check_string:
                self.set_config_value(level1=level1, level2=level2, level3=level3, value=path_value)
            # it is empty
            elif path_value and "/" not in path_value and check_string:
                raise ValueError(
                    f'CONFIG FILE CHECKING FAIL : in section:{level1}, {f"subsection:{level2} value:{level3}" if level3 else f"value:{level2}"}, "{path_value}" is empty')
        elif len(mandatory) > 0:
            raise NotADirectoryError(
                f'CONFIG FILE CHECKING FAIL : in section:{level1}, {f"subsection:{level2} directory:{level3}" if level3 else f"directory:{level2}"}, "{path_value}" {"does not exist" if not Path(path_value).exists() else "is not a valid directory"} but is mandatory for tool: {" ".join(mandatory)}')

    def _check_file_or_string(self, level1, level2, mandatory=(), level3=None, check_string=False):
        """Check if path is a file if not empty
        :return absolute path file"""
        path_value = self.get_config_value(level1=level1, level2=level2, level3=level3)
        path = Path(path_value).resolve().as_posix()
        # it is a path
        if path_value != "" and "/" in path_value:
            if not Path(path).exists() or not Path(path).is_file():
                raise FileNotFoundError(
                    f'CONFIG FILE CHECKING FAIL : in section:{level1}, {f"subsection:{level2}, file {level3}" if level3 else f"file {level2}"}, "{path}" {"does not exist" if not Path(path).exists() else "is not a valid file"}')
            else:
                self.set_config_value(level1=level1, level2=level2, level3=level3, value=path)
        # it is not a path
        elif path_value != "" and "/" not in path_value and check_string:
            self.set_config_value(level1=level1, level2=level2, level3=level3, value=path_value)
        # it is empty
        elif path_value == "" and "/" not in path_value and check_string:
            raise ValueError(
                f'CONFIG FILE CHECKING FAIL : in section:{level1}, {f"subsection:{level2}, value {level3}" if level3 else f"value:{level2}"}, "{path_value}" is empty')
        elif len(mandatory) > 0:
            raise FileNotFoundError(
                f'CONFIG FILE CHECKING FAIL : in  section:{level1} , {f"subsection:{level2}, value {level3}" if level3 else f"value:{level2}"}, "{path_value}" {"does not exist" if not Path(path_value).exists() else "is not a valid file"} but is mandatory for tool: {" ".join(mandatory)}')

    @property
    def export_use_yaml(self):
        """Use to print a dump config.yaml with corrected parameters"""
        def represent_dictionary_order(yamldef, dict_data):
            return yamldef.represent_mapping('tag:yaml.org,2002:map', dict_data.items())

        def setup_yaml():
            yaml.add_representer(OrderedDict, represent_dictionary_order)
        setup_yaml()
        return yaml.dump(self.config, default_flow_style=False, sort_keys=False, indent=4)

    @property
    def string_to_dag(self):
        """ return command line for rule graph """
        return f"""snakemake -s {self.snakefile} {'--use-singularity' if self.use_singularity else ''} {'--use-envmodules' if self.use_env_modules else ''}  --rulegraph"""

    def __repr__(self):
        return f"{self.__class__}({pprint.pprint(self.__dict__)})"


#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import pandas as pd
import argparse
from pathlib import Path
from collections import OrderedDict
from pprint import pprint as pp
version = "0.0.1"
pd.set_option("display.precision", 2)


class AutoVivification(OrderedDict):
    """
    Implementation of perl's autovivification feature.

    Example:

    >>> a = AutoVivification()
    >>> a[1][2][3] = 4
    >>> a[1][3][3] = 5
    >>> a[1][2]['test'] = 6
    >>> print a
    >>> {1: {2: {'test': 6, 3: 4}, 3: {3: 5}}}

    """

    def __getitem__(self, item):
        try:
            return dict.__getitem__(self, item)
        except KeyError:
            value = self[item] = type(self)()
            return value


def main():
    dico_step_files = {}
    for key, list_files in snakemake.params.dico.items():
        dico_step_files[key] = [path.replace("{fastq}",snakemake.params.sample) for path in list_files]
    stat_time = snakemake.output.stat_time
    sample = snakemake.params.sample
    output_dir = Path(snakemake.params.out_dir)
    dico_benchmark_time = AutoVivification()

    for bench_file in dico_step_files["assembly_list"]:
        bench_file_path = Path(bench_file)
        assembler = bench_file_path.stem
        df = pd.read_csv(bench_file, sep="\t")
        dico_benchmark_time["STEP-ASSEMBLERS"][f"ASSEMBLERS"][assembler] = f'{df["s"][0]:.2f}'
     
        # check if MINIASM to load MINIASM_MINIPOLISH file
        if assembler in ("MINIASM"):
            df = pd.read_csv(f"{output_dir}/BENCHMARK/ASSEMBLER/MINIASM_MINIPOLISH.txt", sep="\t")
            df2 = pd.read_csv(f"{output_dir}/BENCHMARK/ASSEMBLER/MINIMAP4MINIASM.txt", sep="\t")
            dico_benchmark_time["STEP-ASSEMBLERS"][f"MINIMAP"][assembler] = f'{df2["s"][0]:.2f}'
            dico_benchmark_time["STEP-ASSEMBLERS"][f"MINIPOLISH"][assembler] = f'{df["s"][0]:.2f}'
        
        # check if SMARTDENOVO to load FASTQ2FASTA file
        if assembler in ("SMARTDENOVO", "SHASTA"):
            df = pd.read_csv(f"{output_dir}/BENCHMARK/ASSEMBLER/FASTQ2FASTA.txt", sep="\t")
            dico_benchmark_time["STEP-ASSEMBLERS"][f"FASTQ2FASTA"][assembler] = f'{df["s"][0]:.2f}'

    if "polishers_list" in dico_step_files:
        for bench_file in dico_step_files["polishers_list"]:
            bench_file_path = Path(bench_file)
            assembler, polisher = bench_file_path.stem.split("_")
            df = pd.read_csv(bench_file, sep="\t")
            dico_benchmark_time["STEP-POLISHER"][f"{polisher}"][assembler] = f'{df["s"][0]:.2f}'

    if "correction_list" in dico_step_files:
        for bench_file in dico_step_files["correction_list"]:
            bench_file_path = Path(bench_file)
            assembler = bench_file_path.stem
            corrector = bench_file_path.parent.stem          
            # check if NANOPOLISH or MEDAKA to load all step file
            if corrector in ("NANOPOLISH", "MEDAKA"):
                summ_step = 0
                summ_variant = 0
                
                variant_list = list(bench_file_path.parent.glob(f"{assembler}*-VARIANTS.txt"))
                # print(f"\n\n{[elm.name for elm in variant_list]}{[elm.parent for elm in variant_list][0]}")
                for bench_file_glob in variant_list:
                    df = pd.read_csv(bench_file_glob, sep="\t")
                    summ_variant += df["s"][0]                
                mean_time = summ_variant/len(variant_list)
                for bench_file_glob in bench_file_path.parent.glob(f"{assembler}*.txt"):
                    if bench_file not in variant_list:
                        df = pd.read_csv(bench_file_glob, sep="\t")
                        summ_step += df["s"][0]
                dico_benchmark_time["STEP-CORRECTION"][f"{corrector}"][assembler] = f'{mean_time+summ_step:.2f}'

    df=pd.DataFrame.from_dict(dico_benchmark_time)
    dataframe_benchmark= df.T.stack().apply(pd.Series)
    with open(stat_time, "w") as benchmark_file:
        with pd.option_context('display.float_format', '{:0.2f}'.format):
        # print(f"dico_benchmark_time:\n{dataframe_benchmark}\n")
            dataframe_benchmark.to_csv(benchmark_file, index=True)


if __name__ == '__main__':
    main()



import click
from shutil import copyfile
from .global_variable import *
from .usefull_function import package_name
import os
import re

def rewrite_if_bind(snakemake_other):
    """function to process click.UNPROCESSED to rewrite --bind params"""
    bind_args = list(filter(re.compile(".*--bind.*").match, snakemake_other))  # Try to select --bind
    if bind_args:
        bind_args_rewrite = f'"--bind {bind_args[0].split(" ")[1]}"'
        snakemake_other_list = list(filter(lambda x: x not in bind_args[0], snakemake_other))  # remove value to rewrite
        snakemake_other_list.append(bind_args_rewrite)
        return snakemake_other_list
    else:
        return snakemake_other

@click.command("run_cluster", short_help='Run workflow on HPC', context_settings=dict(ignore_unknown_options=True, max_content_width=800), no_args_is_help=True)
@click.option('--config', '-c', type=click.Path(exists=True, file_okay=True, readable=True, resolve_path=True), required=True, show_default=True, help=f'Configuration file for run {package_name()}')
@click.option('--pdf', '-pdf', is_flag=True, required=False, default=False, show_default=True, help='Run snakemake with --dag, --rulegraph and --filegraph')
@click.argument('snakemake_other', nargs=-1, type=click.UNPROCESSED)
def run_cluster(config, clusterconfig, pdf, snakemake_other):
    """
    \b
    Run snakemake command line with mandatory parameters.
    SNAKEMAKE_OTHER: You can also pass additional Snakemake parameters
    through this arguments command using the same syntax.
    This will take precedence over Snakemake parameters defined in the profile.
    See: https://snakemake.readthedocs.io/en/stable/executing/cli.html

    Example:
        culebrONT run_cluster -c config.yaml --dry-run --jobs 200
    """
    profile = DEFAULT_PROFILE
    tools = GIT_TOOLS_PATH
    # get user arguments
    click.secho(f'    Profile file: {profile}', fg='yellow')
    click.secho(f'    Config file: {config}', fg='yellow')

    if USER_CLUSTER_CONFIG.exists():
        clusterconfig = USER_CLUSTER_CONFIG
    else:
        click.secho(f"    Please run command line '{package_name()} edit_cluster_config' before the first run of {package_name()} see {DOCS}", fg="red", err=True)
        exit()
    cmd_clusterconfig = f"--cluster-config {clusterconfig}"
    click.secho(f'    Cluster config file load: {clusterconfig}', fg='yellow')

    if tools != GIT_TOOLS_PATH.as_posix():
        ARGS_TOOLS_PATH.parent.mkdir(parents=True, exist_ok=True)
        copyfile(tools, ARGS_TOOLS_PATH)
    elif USER_TOOLS_PATH.exists():
        tools = USER_TOOLS_PATH
    click.secho(f'    Tools Path file: {tools}', fg='yellow')

    cmd_snakemake_base = f"snakemake --show-failed-logs -p -s {SNAKEFILE} --configfile {config} --profile {profile} {cmd_clusterconfig} {' '.join(rewrite_if_bind(snakemake_other))}"
    click.secho(f"\n    {cmd_snakemake_base}\n", fg='bright_blue')
    # exit()
    os.system(cmd_snakemake_base)

    if pdf:
        dag_cmd_snakemake = f"{cmd_snakemake_base} --dag | dot -Tpdf > schema_pipeline_dag.pdf"
        click.secho(f"    {dag_cmd_snakemake}\n", fg='bright_blue')
        os.system(dag_cmd_snakemake)
        rulegraph_cmd_snakemake = f"{cmd_snakemake_base} --rulegraph | dot -Tpdf > schema_pipeline_global.pdf"
        click.secho(f"    {rulegraph_cmd_snakemake}\n", fg='bright_blue')
        os.system(rulegraph_cmd_snakemake)
        filegraph_cmd_snakemake = f"{cmd_snakemake_base} --filegraph | dot -Tpdf > schema_pipeline_files.pdf"
        click.secho(f"    {filegraph_cmd_snakemake}\n", fg='bright_blue')
        os.system(filegraph_cmd_snakemake)


@click.command("run_local", short_help='Run workflow on local computer (use singularity mandatory)', context_settings=dict(ignore_unknown_options=True, max_content_width=800),
               no_args_is_help=True)
@click.option('--config', '-c', type=click.Path(exists=True, file_okay=True, readable=True, resolve_path=True), required=True, help=f'Configuration file for run {package_name()}')
@click.option('--threads', '-t', type=int, required=True, help='Number of threads')
@click.option('--pdf', '-p', is_flag=True, required=False, default=False, help='Run snakemake with --dag, --rulegraph and --filegraph')
@click.argument('snakemake_other', nargs=-1, type=click.UNPROCESSED)
def run_local(config, threads, pdf, snakemake_other):
    """
    \b
    Run snakemake command line with mandatory parameters.
    SNAKEMAKE_OTHER: You can also pass additional Snakemake parameters (check it https://snakemake.readthedocs.io/en/stable/executing/cli.html)
    through this arguments command using the same syntax.
    This will take precedence over Snakemake parameters defined in the profile.
    See: https://snakemake.readthedocs.io/en/stable/executing/cli.html

    Example:
    \b
        culebrONT run_local -c config.yaml --threads 8 --dry-run
        culebrONT run_local -c config.yaml --threads 8 --singularity-args '--bind /mnt:/mnt'
        # in LOCAL using 6 threads for Canu assembly from the total 8 threads
        culebrONT run_local -c config.yaml --threads 8 --set-threads run_canu=6
    """
    click.secho(f'    Config file: {config}', fg='yellow')

    cmd_snakemake_base = f"snakemake --latency-wait 1296000 --cores {threads} --use-singularity --show-failed-logs --printshellcmds -s {SNAKEFILE} --configfile {config}  {' '.join(rewrite_if_bind(snakemake_other))}"
    click.secho(f"\n    {cmd_snakemake_base}\n", fg='bright_blue')
    os.system(cmd_snakemake_base)
    if pdf:
        dag_cmd_snakemake = f"{cmd_snakemake_base} --dag | dot -Tpdf > schema_pipeline_dag.pdf"
        click.secho(f"    {dag_cmd_snakemake}\n", fg='bright_blue')
        os.system(dag_cmd_snakemake)
        rulegraph_cmd_snakemake = f"{cmd_snakemake_base} --rulegraph | dot -Tpdf > schema_pipeline_global.pdf"
        click.secho(f"    {rulegraph_cmd_snakemake}\n", fg='bright_blue')
        os.system(rulegraph_cmd_snakemake)
        filegraph_cmd_snakemake = f"{cmd_snakemake_base} --filegraph | dot -Tpdf > schema_pipeline_files.pdf"
        click.secho(f"    {filegraph_cmd_snakemake}\n", fg='bright_blue')
        os.system(filegraph_cmd_snakemake)



import sys
from pathlib import Path
from .usefull_function import package_name
from ..global_variables import *

# Hack for build docs with unspecified path install
args = str(sys.argv)
if "sphinx" in args:
    INSTALL_PATH = Path(f"/Path/to/{package_name()}_install/")
else:
    INSTALL_PATH = Path(__file__).resolve().parent.parent
SNAKEFILE = INSTALL_PATH.joinpath("snakefiles", "Snakefile")
INSTALL_MODE = INSTALL_PATH.joinpath(".mode.txt")
SNAKEMAKE_SCRIPTS = INSTALL_PATH.joinpath("snakemake_scripts")
DEFAULT_PROFILE = INSTALL_PATH.joinpath("default_profile")
GIT_CONFIG_PATH = INSTALL_PATH.joinpath("install_files", "config.yaml")

GIT_TOOLS_PATH = INSTALL_PATH.joinpath("install_files", "tools_path.yaml")
USER_TOOLS_PATH = Path(f"~/.config/{package_name()}/tools_path.yaml").expanduser()
ARGS_TOOLS_PATH = Path(f"~/.config/{package_name()}/tools_path_args.yaml").expanduser()

GIT_CLUSTER_CONFIG = DEFAULT_PROFILE.joinpath("cluster_config.yaml")
USER_CLUSTER_CONFIG = Path(f"~/.config/{package_name()}/cluster_config.yaml").expanduser()
ARGS_CLUSTER_CONFIG = Path(f"~/.config/{package_name()}/cluster_config_args.yaml").expanduser()



import pandas as pd
import argparse
from pathlib import Path
from collections import OrderedDict
from pprint import pprint as pp
import re
version = "0.0.1"

import pandas as pd
from pathlib import Path


def main():
    dir = snakemake.params.dir
    #print(dir)
    output = snakemake.output.csv
    rep = Path(dir)
    rows = []
    paths = Path(rep).glob('*-version.txt')
    for path in sorted(paths):
        file_name = path.stem.split('-')
        tool = file_name[0].split('_')[0]
        dep = file_name[0].split('_')[1] if len(file_name[0].split('_')) > 1 else '-'
        filev = str(path)
        with open(filev, "r") as fd:
            rows.append([tool, dep, fd.readline().rstrip()])
    df = pd.DataFrame(rows, columns=["Tool", "Secondary", "Version"])
    #print(df)
    df.to_csv(output, index=True)


if __name__ == '__main__':
    main()



import click
from shutil import copyfile
from .global_variable import *
from .usefull_function import get_install_mode, package_name


@click.command("edit_tools", short_help='Edit own tools version', no_args_is_help=False)
@click.option('--restore', '-r', is_flag=True, required=False, default=False, show_default=True, help='Restore default tools_config.yaml (from install)')
def edit_tools(restore):
    if restore:
        if USER_TOOLS_PATH.exists():
            USER_TOOLS_PATH.unlink()
            click.secho(f"\n    Success remove your own tools_path.yaml on path '{USER_TOOLS_PATH}'", fg="yellow")
            click.secho(f"    {package_name()} used '{GIT_TOOLS_PATH}' at default now !!\n\n", fg="bright_red")
        else:
            click.secho(f"    {package_name()} already used default tools_path.yaml '{GIT_TOOLS_PATH}'!!\n\n", fg="red")
    else:
        if not USER_TOOLS_PATH.exists():
            USER_TOOLS_PATH.parent.mkdir(parents=True, exist_ok=True)
            copyfile(GIT_TOOLS_PATH, USER_TOOLS_PATH)
        click.edit(require_save=True, extension='.yaml', filename=USER_TOOLS_PATH)
        click.secho(f"\n    Success install your own tools_path.yaml on path '{USER_TOOLS_PATH}'", fg="yellow")
        click.secho(f"    {package_name()} used '{USER_TOOLS_PATH}' at default now !!\n\n", fg="bright_red")


@click.command("create_config", short_help='Create config.yaml for run', no_args_is_help=True)
@click.option('--configyaml', '-c', default=None,
              type=click.Path(exists=False, file_okay=True, dir_okay=False, readable=True, resolve_path=True),
              required=True, show_default=True, help='Path to create config.yaml')
def create_config(configyaml):
    configyaml = Path(configyaml)
    configyaml.parent.mkdir(parents=True, exist_ok=True)
    copyfile(GIT_CONFIG_PATH.as_posix(), configyaml.as_posix())
    click.edit(require_save=True, extension='.yaml', filename=configyaml)
    click.secho(f"\n    Success create config file on path '{configyaml}'\n    add to command:", fg="yellow")
    mode = get_install_mode()
    click.secho(f"    {package_name()} {'run_cluster' if mode == 'cluster' else 'run_local'} --config {configyaml}\n\n", fg="bright_blue")


@click.command("edit_cluster_config", short_help='Edit cluster_config.yaml use by profile', no_args_is_help=False)
def edit_cluster_config():
    f"""Edit file on {USER_CLUSTER_CONFIG}"""
    USER_CLUSTER_CONFIG.parent.mkdir(parents=True, exist_ok=True)
    if not USER_CLUSTER_CONFIG.exists():
        copyfile(GIT_CLUSTER_CONFIG.as_posix(), USER_CLUSTER_CONFIG.as_posix())
    click.edit(require_save=True, extension='.yaml', filename=USER_CLUSTER_CONFIG.as_posix())
    click.secho(f"\n    See '{DOCS}' to adapt on your cluster", fg="yellow")
    click.secho(f"\n    Success edit cluster_config file on path '{USER_CLUSTER_CONFIG}'", fg="yellow")



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
import os
from pathlib import Path
from setuptools import setup, find_packages
# add for remove error with pip install -e . with pyproject.toml
import site
import sys
site.ENABLE_USER_SITE = "--user" in sys.argv[1:]

NAME = 'culebrONT'
URL = 'https://github.com/SouthGreenPlatform/culebrONT'
CURRENT_PATH = Path(__file__).resolve().parent
VERSION = "2.0.0"

__doc__ = """CulebrONT is an open-source, scalable, modular and traceable Snakemake pipeline, able to launch multiple
assembly tools in parallel, giving you the possibility of circularise, polish, and correct assemblies, checking quality.
CulebrONT can help to choose the best assembly between all possibilities. """



def main():
    setup(
        # Project information
        name=NAME,
        version=VERSION,
        url=URL,
        project_urls={
            'Bug Tracker': f'{URL}/issues',
            'Documentation': f'https://{NAME}-pipeline.readthedocs.io/en/latest/',
            'Source Code': URL
        },
        download_url=f'{URL}/archive/{VERSION}.tar.gz',
        author='''Julie Orjuela (IRD),
                   Aurore Comte (IRD),
                   Sebastien Ravel (CIRAD),
                   Florian Charriat (INRAE),
                   Bao Tram Vi (IRD),
                   François Sabot (IRD)
                   Sebastien Cunnac (IRD)''',
        author_email='sebastien.ravel@cirad.fr',
        description=__doc__.replace("\n",""),
        long_description=CURRENT_PATH.joinpath('README.rst').open('r', encoding='utf-8').read(),
        long_description_content_type='text/x-rst',
        license='GPLv3',

        # docs compilation utils
        command_options={
            'build_sphinx': {
                'project': ('setup.py', NAME),
                'version': ('setup.py', VERSION),
                'release': ('setup.py', VERSION),
                'source_dir': ('setup.py', CURRENT_PATH.joinpath('docs','source').as_posix()),
                'build_dir': ('setup.py', CURRENT_PATH.joinpath('docs','build').as_posix()),
            }},

        # Package information
        packages=find_packages(),
        include_package_data=True,
        # use_scm_version=True,
        use_scm_version={
            "version_scheme": 'release-branch-semver',
            'local_scheme': "node-and-date",
            'normalize' : True,
            "root": ".",
            "relative_to": __file__,
            "fallback_version": VERSION,
            "write_to": 'culebrONT/_version.py',
        },
        setup_requires=['setuptools_scm'],
        python_requires='>=3.6',
        install_requires=[
            'PyYAML',
            'pandas',
            'matplotlib',
            'tabulate',
            'rpy2',
            'ipython',
            'biopython',
            'pysam',
            'numpy',
            'argparse',
            'snakemake',
            'tqdm',
            'click>=8.0.3',
            'cookiecutter'
        ],
        extras_require={
            'docs': ['sphinx_copybutton',
                     'sphinx_rtd_theme',
                     'sphinx_click'],
            'dev': ['tox'],
        },
        entry_points={
            f'{NAME}': [f'{NAME} = __init__'],
            'console_scripts': [f'{NAME} = {NAME}.main:main']},

        # Pypi information
        platforms=['unix', 'linux'],
        keywords=[
            'snakemake',
            'assembly',
            'workflow'
        ],
        classifiers=[
            'License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)',
            'License :: CeCILL-C Free Software License Agreement (CECILL-C)',
            'License :: Free for non-commercial use',
            'Development Status :: 5 - Production/Stable',
            'Intended Audience :: Developers',
            'Intended Audience :: End Users/Desktop',
            'Operating System :: POSIX :: Linux',
            'Programming Language :: Python :: 3.6',
            'Programming Language :: Python :: 3.7',
            'Programming Language :: Python :: 3.8',
            'Programming Language :: Python :: 3.9',
            'Programming Language :: R',
            'Natural Language :: English',
            'Topic :: Scientific/Engineering',
            'Topic :: Scientific/Engineering :: Bio-Informatics',
            'Topic :: Software Development :: Assemblers'
        ],
        options={
            'bdist_wheel': {'universal': True}
        },
        zip_safe=True,  # Don't install the lib as an .egg zipfile
    )


if __name__ == '__main__':
    main()



from .snakeWrapper import SnakeWrapper
from .usefull_function import *
from .global_variable import *
from .edit_files import *
from .install import *
from .run import *
from ..global_variables import *